@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2007-2009 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,15 +16,7 @@
  * 
  */
 
-#include "system.h"
-#include "redir.h"
-#include "syserr.h"
-#include "radius.h"
-#include "dhcp.h"
 #include "chilli.h"
-#include "cmdline.h"
-#include "options.h"
-#include "cmdsock.h"
 
 int
 cmdsock_init() {
@@ -49,6 +41,13 @@ cmdsock_init() {
 	log_err(errno, "could listen to UNIX Socket!");
 	close(cmdsock);
 	cmdsock = -1;
+      } else {
+	if (_options.uid) {
+	  if (chown(_options.cmdsocket, _options.uid, _options.gid)) {
+	    log_err(errno, "could not chown() %s",
+		    _options.cmdsocket);
+	  }
+	}
       }
     }
   }
