@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2009 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,10 +38,13 @@
 #define IPPOOL_NOIP6
 
 #include "system.h"
+#include "lookup.h"
 
 struct ippoolm_t;                /* Forward declaration */
 
 struct ippool_t {
+  int dynsize;                   /* Total number of dynamic addresses */
+  int statsize;                  /* Total number of static addresses */
   int listsize;                  /* Total number of addresses */
   int allowdyn;                  /* Allow dynamic IP address allocation */
   int allowstat;                 /* Allow static IP address allocation */
@@ -64,9 +67,8 @@ struct ippoolm_t {
 #else
   struct in_addr addr;           /* IP address of this member */
 #endif
-  char in_use:1;                 /* 0=available; 1= used */
-  char is_static:1;              /* 0= dynamic; 1 = static */
-  char is_reserved:1;            /* 0= normal; 1 = reserved */
+  char in_use;                   /* 0=available; 1= used */
+  char is_static;                /* 0= dynamic; 1 = static */
   struct ippoolm_t *nexthash;    /* Linked list part of hash table */
   struct ippoolm_t *prev, *next; /* Linked list of free dynamic or static */
   void *peer;                    /* Pointer to peer protocol handler */
@@ -105,6 +107,8 @@ extern int ippool_aton(struct in_addr *addr, struct in_addr *mask,
 
 int ippool_hashadd(struct ippool_t *this, struct ippoolm_t *member);
 
+int ippool_print(int fd, struct ippool_t *this);
+
 #ifndef IPPOOL_NOIP6
 extern uint32_t ippool_hash6(struct in6_addr *addr);
 extern int ippool_getip6(struct ippool_t *this, struct in6_addr *addr);
