@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2007-2009 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,17 +18,7 @@
 
 #define MAIN_FILE
 
-#include "system.h"
-#include "syserr.h"
-#include "cmdline.h"
-#include "dhcp.h"
-#include "radius.h"
-#include "radius_chillispot.h"
-#include "radius_wispr.h"
-#include "redir.h"
 #include "chilli.h"
-#include "options.h"
-#include "cmdsock.h"
 
 struct options_t _options;
 
@@ -39,11 +29,77 @@ static const char *description =
 
 static const char *copyright = 
   "Copyright (c) 2003-2005 Mondru AB., 2006-2010 Coova Technologies LLC, and others.\n"
-  "Licensed under the Gnu General Public License (GPL).\n";
+  "Licensed under the GNU General Public License (GPL).\n";
 
-static const char *usage = \
+static const char *usage =
   "Usage: chilli [OPTIONS]...\n";
 
+static const char *compile_options = "Compiled with "
+#ifdef ENABLE_BINSTATFILE
+  "ENABLE_BINSTATFILE "
+#endif
+#ifdef ENABLE_CHILLIPROXY
+  "ENABLE_CHILLIPROXY "
+#endif
+#ifdef ENABLE_CHILLIRADSEC
+  "ENABLE_CHILLIRADSEC "
+#endif
+#ifdef ENABLE_CHILLIREDIR
+  "ENABLE_CHILLIREDIR "
+#endif
+#ifdef ENABLE_CHILLIXML
+  "ENABLE_CHILLIXML "
+#endif
+#ifdef ENABLE_EWTAPI
+  "ENABLE_EWTAPI "
+#endif
+#ifdef ENABLE_IEEE8021Q
+  "ENABLE_IEEE8021Q "
+#endif
+#ifdef ENABLE_IEEE8023
+  "ENABLE_IEEE8023 "
+#endif
+#ifdef ENABLE_JSON
+  "ENABLE_JSON "
+#endif
+#ifdef ENABLE_LARGELIMITS
+  "ENABLE_LARGELIMITS "
+#endif
+#ifdef ENABLE_LEAKYBUCKET
+  "ENABLE_LEAKYBUCKET "
+#endif
+#ifdef ENABLE_MINIPORTAL
+  "ENABLE_MINIPORTAL "
+#endif
+#ifdef ENABLE_NETNAT
+  "ENABLE_NETNAT "
+#endif
+#ifdef ENABLE_PPPOE
+  "ENABLE_PPPOE "
+#endif
+#ifdef ENABLE_PROXYVSA
+  "ENABLE_PROXYVSA "
+#endif
+#ifdef ENABLE_RTMON
+  "ENABLE_RTMON "
+#endif
+#ifdef ENABLE_SESSGARDEN
+  "ENABLE_SESSGARDEN "
+#endif
+#ifdef ENABLE_STATFILE
+  "ENABLE_STATFILE "
+#endif
+#ifdef HAVE_MATRIXSSL
+  "HAVE_MATRIXSSL "
+#endif
+#ifdef HAVE_OPENSSL
+  "HAVE_OPENSSL "
+#endif
+#ifdef USING_POLL
+  "USING_POLL "
+#endif
+;
+
 extern const char *gengetopt_args_info_help[];
 
 static void
@@ -62,6 +118,8 @@ options_print_help (void) {
   while (gengetopt_args_info_help[i])
     printf("%s\n", gengetopt_args_info_help[i++]);
 
+  printf("\n%s\n", compile_options);
+
   printf("\n%s\n", copyright);
 }
 
@@ -184,6 +242,40 @@ int main(int argc, char **argv) {
   _options.domaindnslocal = args_info.domaindnslocal_flag;
   _options.framedservice = args_info.framedservice_flag;
   _options.radsec = args_info.radsec_flag;
+  _options.proxymacaccept = args_info.proxymacaccept_flag;
+
+#ifdef ENABLE_LEAKYBUCKET
+  _options.bwbucketupsize = args_info.bwbucketupsize_arg;
+  _options.bwbucketdnsize = args_info.bwbucketdnsize_arg;
+  _options.bwbucketminsize = args_info.bwbucketminsize_arg;
+#endif
+
+#ifdef ENABLE_PROXYVSA
+  if (args_info.proxylocattr_given) {
+    for (numargs = 0; numargs < args_info.proxylocattr_given 
+	   && numargs < PROXYVSA_ATTR_CNT; ++numargs)  {
+      unsigned int i[2];
+
+      switch (sscanf(args_info.proxylocattr_arg[numargs], 
+		     "%u,%u", &i[0], &i[1])) {
+      case 0:
+	log_err(0, "invalid input %s", args_info.proxylocattr_arg[numargs]);
+	break;
+      case 1:
+	_options.proxy_loc[numargs].attr = i[0];
+	break;
+      case 2:
+	_options.proxy_loc[numargs].attr_vsa = i[0];
+	_options.proxy_loc[numargs].attr = i[1];
+	break;
+      }
+      
+      log_dbg("Proxy location attr %d %d", 
+	      (int)_options.proxy_loc[numargs].attr_vsa, 
+	      (int)_options.proxy_loc[numargs].attr);
+    }
+  }
+#endif
 
   if (args_info.dhcpgateway_arg &&
       !inet_aton(args_info.dhcpgateway_arg, &_options.dhcpgwip)) {
@@ -632,11 +724,9 @@ int main(int argc, char **argv) {
 
   memset(_options.macok, 0, sizeof(_options.macok));
   _options.macoklen = 0;
+
   for (numargs = 0; numargs < args_info.macallowed_given; ++numargs) {
-    if (_options.debug & DEBUG_CONF) {
-      log_dbg("Macallowed #%d: %s\n", numargs, 
-	      args_info.macallowed_arg[numargs]);
-    }
+
     char *p1 = NULL;
     char *p2 = NULL;
     char *p3 = malloc(strlen(args_info.macallowed_arg[numargs])+1);
@@ -644,6 +734,8 @@ int main(int argc, char **argv) {
 
     unsigned int mac[6];
 
+    log_dbg("Macallowed #%d: %s", numargs, args_info.macallowed_arg[numargs]);
+
     strcpy(p3, args_info.macallowed_arg[numargs]);
     p1 = p3;
     if ((p2 = strchr(p1, ','))) {
@@ -665,11 +757,9 @@ int main(int argc, char **argv) {
 	}
 	else {
 
-	  if (_options.debug & DEBUG_CONF) {
-	    log_dbg("Macallowed address #%d: %.2X-%.2X-%.2X-%.2X-%.2X-%.2X\n", 
-		   _options.macoklen,
-		   mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
-	  }
+	  log_dbg("Macallowed address #%d: %.2X-%.2X-%.2X-%.2X-%.2X-%.2X", 
+		  _options.macoklen,
+		  mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
 
 	  for (i = 0; i < 6; i++)
 	    _options.macok[_options.macoklen][i] = (unsigned char) mac[i]; 
@@ -681,7 +771,7 @@ int main(int argc, char **argv) {
       if (p2) {
 	p1 = p2+1;
 	if ((p2 = strchr(p1, ','))) {
-	  *p2 = '\0';
+	  *p2 = 0;
 	}
       }
       else {
@@ -726,6 +816,11 @@ int main(int argc, char **argv) {
   _options.unixipc = STRDUP(args_info.unixipc_arg);
 #endif
 
+#ifdef HAVE_NETFILTER_COOVA
+  if (_options.kname) free(_options.kname);
+  _options.kname = STRDUP(args_info.kname_arg);
+#endif
+
   if (_options.uamurl) free(_options.uamurl);
   _options.uamurl = STRDUP(args_info.uamserver_arg);
 
