@@ -18,19 +18,7 @@
 
 #define MAIN_FILE
 
-#include "system.h"
-#include "syserr.h"
-#include "cmdline.h"
-#include "dhcp.h"
-#include "radius.h"
-#include "radius_chillispot.h"
-#include "radius_wispr.h"
-#include "redir.h"
 #include "chilli.h"
-#include "options.h"
-#include "cmdsock.h"
-#include "md5.h"
-#include "conn.h"
 
 #ifdef USING_CURL
 #include <curl/curl.h>
@@ -40,7 +28,7 @@
 
 /*
  * Plans (todo):
- *  - "Chilli Dog" will provide a simple RADIUS->HTTP AAA proxy (loosly based on WiFiDog). 
+ *  - Provide a simple RADIUS->HTTP AAA proxy (loosly based on WiFiDog). 
  *  - It should also be able to proxy to an alternate RADIUS server(s). 
  *  - It should also be able to establish and use a RadSec Tunnel. 
  *
@@ -339,7 +327,6 @@ static int bstring_data(void *ptr, size_t size, size_t nmemb, void *userdata) {
 static int http_aaa_setup(struct radius_t *radius, proxy_request *req) {
   int result = -2;
   CURL *curl;
-  CURLcode res;
 
   char *user = 0;
   char *pwd = 0;
@@ -351,14 +338,12 @@ static int http_aaa_setup(struct radius_t *radius, proxy_request *req) {
   req->radius = radius;
 
   if ((curl = req->curl = curl_easy_init()) != NULL) {
-    struct curl_httppost *formpost=NULL;
-    struct curl_httppost *lastptr=NULL;
-    char error_buffer[CURL_ERROR_SIZE + 1];
+    char error_buffer[CURL_ERROR_SIZE];
 
-    memset(&error_buffer, 0, sizeof(error_buffer));
+    memset(error_buffer, 0, sizeof(error_buffer));
 
     if (req->post) {
-      curl_easy_setopt(curl, CURLOPT_POSTFIELDS, req->post->data);
+      curl_easy_setopt(curl, CURLOPT_POSTFIELDS, (char *) req->post->data);
     }
 
     if (user && pwd) {
@@ -419,10 +404,10 @@ static int http_aaa_setup(struct radius_t *radius, proxy_request *req) {
     curl_easy_setopt(curl, CURLOPT_NETRC, 0);
     curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
 
-    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, bstring_data);
-    curl_easy_setopt(curl, CURLOPT_WRITEDATA, req->data);
+    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, (curl_write_callback) bstring_data);
+    curl_easy_setopt(curl, CURLOPT_WRITEDATA, (char *) req->data);
 
-    curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, &error_buffer);
+    curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, error_buffer);
 
     result = 0;
   }
@@ -477,12 +462,12 @@ static int http_aaa_setup(struct radius_t *radius, proxy_request *req) {
     req->wbuf = bfromcstr("");
 
   bassignformat(req->wbuf, 
-		"GET %s HTTP/1.1\n"
-		"Host: %s\n"
-		"User-Agent: CoovaChilli " VERSION "\n"
-		"Connection: close\n"
-		"\n",
-		req->url->data + uripos, hostname);
+		"GET %s HTTP/1.1\r\n"
+		"Host: %s:%d\r\n"
+		"User-Agent: CoovaChilli " VERSION "\r\n"
+		"Connection: close\r\n"
+		"\r\n",
+		req->url->data + uripos, hostname, port);
 
   if (conn_setup(&req->conn, hostname, port, req->wbuf))
     return -1;
@@ -581,7 +566,8 @@ static void http_aaa_register(int argc, char **argv, int i) {
     }
   }
 
-  redir_md_param(req.url, _options.uamsecret, "&");
+  if (_options.uamsecret && _options.uamsecret[0])
+    redir_md_param(req.url, _options.uamsecret, "&");
 
 #ifdef USING_CURL
   curl_global_init(CURL_GLOBAL_ALL);
@@ -795,30 +781,30 @@ static void process_radius(struct radius_t *radius, struct radius_packet_t *pack
 
     if (!radius_getattr(pack, &attr, RADIUS_ATTR_ACCT_SESSION_TIME, 0,0,0)) {
       uint32_t val = ntohl(attr->v.i);
-      bassignformat(tmp, "&duration=%d", val);
+      bassignformat(tmp, "&duration=%ld", (long) val);
       bconcat(req->url, tmp);
     }
 
     if (!radius_getattr(pack, &attr, RADIUS_ATTR_VENDOR_SPECIFIC,
 		   RADIUS_VENDOR_CHILLISPOT, RADIUS_ATTR_CHILLISPOT_VLAN_ID, 0)) {
       uint32_t val = ntohl(attr->v.i);
-      bassignformat(tmp, "&vlan=%d", val);
+      bassignformat(tmp, "&vlan=%ld", (long) val);
       bconcat(req->url, tmp);
     }
 
     if (!radius_getattr(pack, &attr, RADIUS_ATTR_ACCT_INPUT_OCTETS, 0,0,0)) {
       char *direction = _options.swapoctets ? "up" : "down";
-      uint64_t val = ntohl(attr->v.i);
+      uint64_t val = (uint64_t) ntohl(attr->v.i);
       uint64_t input = val;
       if (!radius_getattr(pack, &attr, RADIUS_ATTR_ACCT_INPUT_GIGAWORDS, 0,0,0)) {
 	val = (uint64_t) ntohl(attr->v.i);
 	input |= (val << 32);
       }
-      bassignformat(tmp, "&bytes_%s=%ld", direction, input);
+      bassignformat(tmp, "&bytes_%s=%lld", direction, (long long) input);
       bconcat(req->url, tmp);
       if (!radius_getattr(pack, &attr, RADIUS_ATTR_ACCT_INPUT_PACKETS, 0,0,0)) {
 	uint32_t sval = ntohl(attr->v.i);
-	bassignformat(tmp, "&pkts_%s=%ld", direction, sval);
+	bassignformat(tmp, "&pkts_%s=%ld", direction, (long) sval);
 	bconcat(req->url, tmp);
       }
     }
@@ -831,11 +817,11 @@ static void process_radius(struct radius_t *radius, struct radius_packet_t *pack
 	val = (uint64_t) ntohl(attr->v.i);
 	output |= (val << 32);
       }
-      bassignformat(tmp, "&bytes_%s=%ld", direction, output);
+      bassignformat(tmp, "&bytes_%s=%lld", direction, (long long) output);
       bconcat(req->url, tmp);
       if (!radius_getattr(pack, &attr, RADIUS_ATTR_ACCT_OUTPUT_PACKETS, 0,0,0)) {
 	uint32_t sval = ntohl(attr->v.i);
-	bassignformat(tmp, "&pkts_%s=%ld", direction, sval);
+	bassignformat(tmp, "&pkts_%s=%ld", direction, (long) sval);
 	bconcat(req->url, tmp);
       }
     }
@@ -896,7 +882,8 @@ static void process_radius(struct radius_t *radius, struct radius_packet_t *pack
   }
 
   if (!error) {
-    redir_md_param(req->url, _options.uamsecret, "&");
+    if (_options.uamsecret && _options.uamsecret[0])
+      redir_md_param(req->url, _options.uamsecret, "&");
     
     log_dbg("==> %s", req->url->data);
     http_aaa(radius, req);
@@ -973,6 +960,16 @@ int main(int argc, char **argv) {
   radius_set(radius_auth, 0, 0);
   radius_set(radius_acct, 0, 0);
 
+  if (_options.gid && setgid(_options.gid)) {
+    log_err(errno, "setgid(%d) failed while running with gid = %d\n", 
+	    _options.gid, getgid());
+  }
+  
+  if (_options.uid && setuid(_options.uid)) {
+    log_err(errno, "setuid(%d) failed while running with uid = %d\n", 
+	    _options.uid, getuid());
+  }
+
   while (keep_going) {
 
     if (reload_config) {
