@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2009 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2010 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,42 +18,16 @@
 
 #define MAIN_FILE
 
-#include "system.h"
-#include "syserr.h"
-#include "cmdline.h"
-#include "dhcp.h"
-#include "redir.h"
-#include "radius.h"
 #include "chilli.h"
-#include "options.h"
-#include "cmdsock.h"
-#include "md5.h"
-#include "conn.h"
 
 struct options_t _options;
 
+static select_ctx sctx;
+
 #ifndef USING_IPC_UNIX
 #error This requires the UNIX IPC method
 #endif
 
-typedef struct _redir_request {
-  int index;
-  
-  char inuse:1;
-  
-  bstring url;
-  bstring data;
-  bstring post;
-  bstring wbuf;
-
-  struct conn_t conn;
-  
-  int socket_fd;
-
-  struct _redir_request *prev, *next;
-  
-} redir_request;
-
 static int max_requests = 0;
 static redir_request * requests = 0;
 static redir_request * requests_free = 0;
@@ -65,7 +39,7 @@ static redir_request * get_request() {
   if (!max_requests) {
 
     max_requests = 2048; /* hard maximum! (should be configurable) */
-
+    
     requests = (redir_request *) calloc(max_requests, sizeof(redir_request));
     for (i=0; i < max_requests; i++) {
       requests[i].index = i;
@@ -79,6 +53,15 @@ static redir_request * get_request() {
   }
   
   if (requests_free) {
+    if (_options.debug) {
+      int cnt = 0;
+      req = requests_free;
+      while (req) {	
+	req = req->next;
+	cnt++;
+      }
+      log_dbg("redir free connections %d", cnt);;
+    }
     req = requests_free;
     requests_free = requests_free->next;
     if (requests_free)
@@ -87,10 +70,11 @@ static redir_request * get_request() {
   
   if (!req) {
     /* problem */
-    log_err(0,"out of connections\n");
+    log_err(0,"out of connections!");
     return 0;
   }
-  
+
+  req->state = 0;
   req->next = req->prev = 0;
   req->inuse = 1;
   return req;
@@ -99,7 +83,9 @@ static redir_request * get_request() {
 static void close_request(redir_request *req) {
   log_dbg("closing request");
   req->inuse = 0;
+  req->proxy = 0;
   req->socket_fd = 0;
+  req->state = 0;
   if (requests_free) {
     requests_free->prev = req;
     req->next = requests_free;
@@ -120,8 +106,14 @@ sock_redir_getstate(struct redir_t *redir,
   char *statedir = _options.statedir ? _options.statedir : DEFSTATEDIR;
   char filedest[512];
 
-  snprintf(filedest, sizeof(filedest), "%s/%s", statedir, 
-	   _options.unixipc ? _options.unixipc : "chilli.ipc");
+  if (!_options.unixipc) {
+    snprintf(filedest, sizeof(filedest), "%s/chilli.ipc", statedir);
+  } else {
+    if (_options.unixipc[0]=='/')
+      snprintf(filedest, sizeof(filedest), "%s", _options.unixipc);
+    else
+      snprintf(filedest, sizeof(filedest), "%s/%s", statedir, _options.unixipc);
+  }
 
   msg.mtype = REDIR_MSG_STATUS_TYPE;
   memcpy(&msg.mdata.address, address, sizeof(msg.mdata.address));
@@ -168,8 +160,23 @@ sock_redir_getstate(struct redir_t *redir,
 
 static int redir_conn_finish(struct conn_t *conn, void *ctx) {
   redir_request *req = (redir_request *)ctx;
-  conn_close(&req->conn);
+#ifdef HAVE_SSL
+  if (req->sslcon) {
+    openssl_shutdown(req->sslcon, 2);
+    openssl_free(req->sslcon);
+    req->sslcon=0;
+  }
+#endif
+  if (req->conn.sock) {
+    if (req->state & REDIR_CONN_FD) {
+      net_select_rmfd(&sctx, req->conn.sock);
+    }
+    conn_close(&req->conn);
+  }
   if (req->socket_fd) {
+    if (req->state & REDIR_SOCKET_FD) {
+      net_select_rmfd(&sctx, req->socket_fd);
+    }
     close(req->socket_fd);
   }
   close_request(req);
@@ -179,15 +186,18 @@ static int redir_conn_finish(struct conn_t *conn, void *ctx) {
 static int redir_conn_read(struct conn_t *conn, void *ctx) {
   redir_request *req = (redir_request *)ctx;
   char b[PKT_MAX_LEN];
+
   int r = read(conn->sock, b, sizeof(b));
 
-  /*log_dbg("read: %d", r);*/
+  log_dbg("conn_read: %d", r);
 
   if (r <= 0) {
     redir_conn_finish(conn, ctx);
   } else if (r > 0) {
-    int w = write(req->socket_fd, b, r);
-    /*log_dbg("write: %d", w);*/
+    int w;
+    req->last_active = mainclock_tick();
+    w = write(req->socket_fd, b, r);
+    log_dbg("write: %d", w);
     if (r != w) {
       log_err(errno, "problem writing what we read");
       redir_conn_finish(conn, ctx);
@@ -227,13 +237,11 @@ redir_handle_url(struct redir_t *redir,
 		 struct redir_conn_t *conn, 
 		 struct redir_httpreq_t *httpreq,
 		 struct redir_socket_t *socket,
-		 void *ctx) {
-  redir_request *req = get_request();
+		 struct sockaddr_in *peer, 
+		 redir_request *req) {
   int port = 80;
   int i;
 
-  if (!req) return 1;
-
   for (i=0; i < MAX_REGEX_PASS_THROUGHS; i++) {
     
     int matches = 1;
@@ -281,21 +289,20 @@ redir_handle_url(struct redir_t *redir,
     }
 
     if (matches) {
-      req->socket_fd = socket->fd[1];
-      if (!req->wbuf) req->wbuf = bfromcstr("");
-      bassign(req->wbuf, httpreq->data_in);
+      req->proxy = 1;
       
       if (conn_setup(&req->conn, httpreq->host, port, req->wbuf)) {
 	log_err(errno, "conn_setup()");
 	return -1;
       }
+
+      req->state |= REDIR_CONN_FD;
+      net_select_addfd(&sctx, req->conn.sock, SELECT_READ);
       
-      conn_set_readhandler(&req->conn, redir_conn_read, req);
-      conn_set_donehandler(&req->conn, redir_conn_finish, req);
       return 0;
     }
   }
-
+  
   return 1;
 }
 
@@ -319,7 +326,8 @@ int redir_accept2(struct redir_t *redir, int idx) {
 
   addrlen = sizeof(struct sockaddr_in);
 
-  if (getsockname(new_socket, (struct sockaddr *)&baddress, &addrlen) < 0) {
+  if (getsockname(new_socket, (struct sockaddr *)&baddress, 
+		  &addrlen) < 0) {
     log_warn(errno, "getsockname() failed!");
   }
 
@@ -346,33 +354,57 @@ int redir_accept2(struct redir_t *redir, int idx) {
       close(new_socket);
       return 0; 
     }
-
+    
     snprintf(buffer,sizeof(buffer)-1,"%s",inet_ntoa(address.sin_addr));
     setenv("TCPREMOTEIP",buffer,1);
     setenv("REMOTE_ADDR",buffer,1);
     snprintf(buffer,sizeof(buffer)-1,"%d",ntohs(address.sin_port));
     setenv("TCPREMOTEPORT",buffer,1);
     setenv("REMOTE_PORT",buffer,1);
-
+    
     char *binqqargs[2] = { _options.uamui, 0 } ;
     
     execv(*binqqargs, binqqargs);
     
   } else {
-    
-    return redir_main(redir, new_socket, new_socket, &address, &baddress, idx, 0);
 
+    redir_request *req = get_request();
+
+    log_dbg("redir_main() for %s", inet_ntoa(address.sin_addr));
+
+    req->last_active = mainclock_tick();
+    memcpy(&req->conn.peer, &address, sizeof (struct sockaddr_in));
+    memcpy(&req->baddr, &baddress, sizeof (struct sockaddr_in));
+
+    req->uiidx = idx;
+    req->socket_fd = new_socket;
+    if (!req->wbuf) req->wbuf = bfromcstr("");
+    else bassigncstr(req->wbuf, "");
+
+    conn_set_readhandler(&req->conn, redir_conn_read, req);
+    conn_set_donehandler(&req->conn, redir_conn_finish, req);
+    
+    switch (redir_main(redir, new_socket, new_socket,
+		       &address, &baddress, idx, req)) {
+    case 1:
+      log_dbg("redir queued %s", inet_ntoa(address.sin_addr));
+      req->state |= REDIR_SOCKET_FD;
+      net_select_addfd(&sctx, req->socket_fd, SELECT_READ);
+      return 1;
+    case 0: 
+      log_dbg("redir completed %s", inet_ntoa(address.sin_addr));
+      redir_conn_finish(&req->conn, req);
+      return 0;
+    default:
+      redir_conn_finish(&req->conn, req);
+      return -1;
+    }
   }
 
   return 0;
 }
 
 int main(int argc, char **argv) {
-  int maxfd = 0;
-  fd_set fdread;
-  fd_set fdwrite;
-  fd_set fdexcep;
-
   int status;
   int idx;
   int active_last = 0;
@@ -406,7 +438,8 @@ int main(int argc, char **argv) {
   close(fd);
   
   /* create an instance of redir */
-  if (redir_new(&redir, &_options.uamlisten, _options.uamport, _options.uamuiport)) {
+  if (redir_new(&redir, &_options.uamlisten, _options.uamport, 
+		_options.uamuiport)) {
     log_err(0, "Failed to create redir");
     return -1;
   }
@@ -416,19 +449,35 @@ int main(int argc, char **argv) {
     return -1;
   }
 
-  if (redir->fd[0] > maxfd) maxfd = redir->fd[0];
-  if (redir->fd[1] > maxfd) maxfd = redir->fd[1];
   redir_set(redir, hwaddr, (_options.debug));
   redir_set_cb_getstate(redir, sock_redir_getstate);
-
+  
   redir->cb_handle_url = redir_handle_url;
-  redir->cb_handle_url_ctx = 0;
+
+  if (net_select_init(&sctx))
+    log_err(errno, "select init");
+
+  /* epoll */
+  net_select_addfd(&sctx, redir->fd[0], SELECT_READ);
+  net_select_addfd(&sctx, redir->fd[1], SELECT_READ);
+
+  if (_options.gid && setgid(_options.gid)) {
+    log_err(errno, "setgid(%d) failed while running with gid = %d\n", 
+	    _options.gid, getgid());
+  }
+  
+  if (_options.uid && setuid(_options.uid)) {
+    log_err(errno, "setuid(%d) failed while running with uid = %d\n", 
+	    _options.uid, getuid());
+  }
 
   while (keep_going) {
-    FD_ZERO(&fdread);
-    FD_ZERO(&fdwrite);
-    FD_ZERO(&fdexcep);
 
+    /* select/poll */
+    net_select_zero(&sctx);
+    net_select_fd(&sctx, redir->fd[0], SELECT_READ);
+    net_select_fd(&sctx, redir->fd[1], SELECT_READ);
+  
     active = 0;
 
     if (reload_config) {
@@ -438,33 +487,78 @@ int main(int argc, char **argv) {
       redir_set(redir, hwaddr, _options.debug);
     }
 
-    if (redir->fd[0])
-      fd_set(redir->fd[0], &fdread);
-
-    if (redir->fd[1])
-      fd_set(redir->fd[1], &fdread);
-
     for (idx=0; idx < max_requests; idx++) {
-      conn_fd(&requests[idx].conn, &fdread, &fdwrite, &fdexcep, &maxfd);
+
+      conn_select_fd(&requests[idx].conn, &sctx);
+
       if (requests[idx].inuse && requests[idx].socket_fd) {
+	time_t now = mainclock_tick();
 	int fd = requests[idx].socket_fd;
-	if (fd > maxfd) maxfd = fd;
-	fd_set(fd, &fdread);
-	active++;
+	int timeout = 60;
+
+	if (now - requests[idx].last_active > timeout) {
+	  log_dbg("timeout connection %d", idx);
+	  redir_conn_finish(&requests[idx].conn, &requests[idx]);
+	} else {
+	  timeout = 0;
+	  net_select_fd(&sctx, fd, SELECT_READ);
+	  active++;
+	}
+	
+	if (_options.debug) {
+	  struct sockaddr_in address;
+	  socklen_t addrlen = sizeof(address);
+	  
+	  if (getpeername(requests[idx].socket_fd, (struct sockaddr *)&address, 
+			  &addrlen) >= 0) {
+	    char line[512];
+	    
+	    snprintf(line, sizeof(line),
+		     "#%d (%d) %d connection from %s %d",
+		     timeout ? -1 : active, fd, (int) requests[idx].last_active,
+		     inet_ntoa(address.sin_addr),
+		     ntohs(address.sin_port));
+	    
+	    if (requests[idx].conn.sock) {
+	      addrlen = sizeof(address);
+	      if (getpeername(requests[idx].conn.sock, (struct sockaddr *)&address,
+			      &addrlen) >= 0) {
+		snprintf(line+strlen(line), sizeof(line)-strlen(line),
+			 " to %s %d",
+			 inet_ntoa(address.sin_addr),
+			 ntohs(address.sin_port));
+	      }
+	    }
+	    
+	    if (timeout) {
+	      snprintf(line+strlen(line), sizeof(line)-strlen(line),
+		       " (timeout)");
+	    }
+
+	    log_dbg("%s", line);
+	  }
+	}
       }
     }
 
-    /*
-      timeout.tv_sec = 1;
-      timeout.tv_usec = 0;
-    */
-
     if (active != active_last) {
-      log_info("active connections: %d", active);
+      log_dbg("active connections: %d", active);
       active_last = active;
     }
-
-    status = select(maxfd + 1, &fdread, &fdwrite, &fdexcep, NULL/*&timeout*/);
+    
+    status = net_select(&sctx);
+    
+    if (status > 0) {
+      int i;
+      log_dbg("epoll %d", status);
+      for (i=0; i < status; i++) {
+	log_dbg("epoll fd %d %d", 
+		sctx.events[i].data.fd, 
+		sctx.events[i].events);
+      }
+    }
+    /*
+    */
 
     switch (status) {
     case -1:
@@ -473,52 +567,107 @@ int main(int argc, char **argv) {
       }
       break;  
 
-    case 0:
-      break;
-
     default:
       if (status > 0) {
 
 	if (redir->fd[0])
-	  if (fd_isset(redir->fd[0], &fdread) && redir_accept2(redir, 0) < 0)
+	  if (net_select_read_fd(&sctx, redir->fd[0]) && 
+	      redir_accept2(redir, 0) < 0)
 	    log_err(0, "redir_accept() failed!");
-
+	
 	if (redir->fd[1])
-	  if (fd_isset(redir->fd[1], &fdread) && redir_accept2(redir, 1) < 0)
+	  if (net_select_read_fd(&sctx, redir->fd[1]) && 
+	      redir_accept2(redir, 1) < 0)
 	    log_err(0, "redir_accept() failed!");
-
+      
 	for (idx=0; idx < max_requests; idx++) {
-	  conn_update(&requests[idx].conn, &fdread, &fdwrite, &fdexcep);
+	  
+	  conn_select_update(&requests[idx].conn, &sctx);
+	
 	  if (requests[idx].inuse && requests[idx].socket_fd) {
 	    int fd = requests[idx].socket_fd;
-	    if (FD_ISSET(fd, &fdread)) {
-	      char b[1500];
-	      int r;
-
+	    
 #ifdef HAVE_SSL
-#endif
-	      r = read(fd, b, sizeof(b));
-
-	      /*log_dbg("read: %d", r);*/
-	      
-	      if (r <= 0) {
+	    if (requests[idx].sslcon) {
+	      if (openssl_check_accept(requests[idx].sslcon) < 0) {
 		redir_conn_finish(&requests[idx].conn, &requests[idx]);
-	      } else if (r > 0) {
-		int w = write(requests[idx].conn.sock, b, r);
-		/*log_dbg("write: %d", w);*/
-		if (r != w) {
-		  log_err(errno, "problem writing what we read");
+		continue;
+	      }
+	    }
+#endif
+	    
+	    if (net_select_read_fd(&sctx, fd)) {
+	      
+	      if (requests[idx].proxy) {
+		char b[1500];
+		int r;
+		
+#ifdef HAVE_SSL
+		if (requests[idx].sslcon) {
+		  /*
+		  log_dbg("proxy_read_ssl");
+		  */
+		  r = openssl_read(requests[idx].sslcon, 
+				   b, sizeof(b), 0);
+		} else
+#endif
+		  r = recv(fd, b, sizeof(b), 0);
+		
+		/*
+		log_dbg("proxy_read: %d %d", fd, r);
+		*/
+		
+		if (r <= 0) {
+		  
 		  redir_conn_finish(&requests[idx].conn, &requests[idx]);
+		  
+		} else if (r > 0) {
+		  
+		  int w;
+		  requests[idx].last_active = mainclock_tick();
+		  w = write(requests[idx].conn.sock, b, r);
+		  
+		  /*
+		  log_dbg("proxy_write: %d", w);
+		   */
+		  if (r != w) {
+		    log_err(errno, "problem writing what we read");
+		    redir_conn_finish(&requests[idx].conn, &requests[idx]);
+		  }
+		}
+		
+	      } else {
+#ifdef HAVE_SSL
+	      go_again:
+#endif
+		switch (redir_main(redir, fd, fd, 
+				   &requests[idx].conn.peer,
+				   &requests[idx].baddr, 
+				   requests[idx].uiidx, 
+				   &requests[idx])) {
+		case 1:
+#ifdef HAVE_SSL
+		  if (requests[idx].sslcon && 
+		      openssl_pending(requests[idx].sslcon) > 0)
+		    goto go_again;
+#endif
+		  break;
+		case -1: 
+		  log_dbg("redir error");
+		default:
+		  log_dbg("redir completed");
+		  redir_conn_finish(&requests[idx].conn, &requests[idx]);
+		  break;
 		}
 	      }
 	    }
 	  }
 	}
       }
-
+      
       break;
     }
   }
-
+  
   return 0;
 }
