@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2009 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,16 +17,7 @@
  * 
  */
 
-#include "system.h"
-#include "syserr.h"
-#include "radius.h"
-#include "md5.h"
-#include "dhcp.h"
-#include "redir.h"
 #include "chilli.h"
-#include "options.h"
-#include "radius_wispr.h"
-#include "radius_chillispot.h"
 
 #define _debug_ 0
 
@@ -74,13 +65,15 @@ int radius_printqueue(struct radius_t *this) {
 
   for(n=0; n < mx; n++) {
     if (this->queue[n].state) {
-      log_dbg("%3d %3d %3d %3d %8d %8d %d",
-	     n, this->queue[n].state,
-	     this->queue[n].next,
-	     this->queue[n].prev,
-	     (int) this->queue[n].timeout.tv_sec,
-	     (int) this->queue[n].timeout.tv_usec,
-	     (int) this->queue[n].retrans);
+      log_dbg("n=%3d id=%3d state=%3d next=%3d prev=%3d %8d %8d %d",
+	      n, 
+	      this->queue[n].p.id,
+	      this->queue[n].state,
+	      this->queue[n].next,
+	      this->queue[n].prev,
+	      (int) this->queue[n].timeout.tv_sec,
+	      (int) this->queue[n].timeout.tv_usec,
+	      (int) this->queue[n].retrans);
     }
   }
 
@@ -761,6 +754,20 @@ radius_addattr(struct radius_t *this, struct radius_packet_t *pack,
   return 0;
 }
 
+#ifdef ENABLE_PROXYVSA
+int
+radius_addvsa(struct radius_packet_t *pack, struct redir_state *state) {
+  if (state->vsalen) {
+    uint16_t length = ntohs(pack->length);
+    void *m = (void *) pack + (size_t) length;
+    memcpy(m, state->vsa, state->vsalen);
+    length += state->vsalen;
+    pack->length = htons(length);
+    log_dbg("Recalled VSA");
+  }
+  return 0;
+}
+#endif
 
 /* 
  * radius_getattr()
@@ -810,13 +817,13 @@ radius_getnextattr(struct radius_packet_t *pack, struct radius_attr_t **attr,
     if (t->t != type) 
       continue;
     
-    if (t->t == RADIUS_ATTR_VENDOR_SPECIFIC && 
+    if (t->t == RADIUS_ATTR_VENDOR_SPECIFIC && vendor_id &&
 	(ntohl(t->v.vv.i) != vendor_id || t->v.vv.t != vendor_type))
       continue;
     
     if (count == instance) {
       
-      if (type == RADIUS_ATTR_VENDOR_SPECIFIC)
+      if (type == RADIUS_ATTR_VENDOR_SPECIFIC && vendor_id)
 	*attr = (struct radius_attr_t *) &t->v.vv.t;
       else
 	*attr = t;
@@ -1016,6 +1023,8 @@ int radius_pwdecode(struct radius_t *this,
   MD5_CTX context;
   unsigned char output[RADIUS_MD5LEN];
 
+  log_dbg("pw decode secret=%s", secret);
+
   if (srclen > dstsize) {
     log_err(0, "radius_pwdecode srclen larger than dstsize");
     return -1;
@@ -1029,7 +1038,7 @@ int radius_pwdecode(struct radius_t *this,
   *dstlen = srclen;
 
   /*
-  if (this->debug) {
+  if (_options.debug) {
     printf("pwdecode srclen %d\n", srclen);
     for (n=0; n< srclen; n++) {
       printf("%.2x ", src[n]);
@@ -1045,14 +1054,6 @@ int radius_pwdecode(struct radius_t *this,
 	printf("\n");
     }
     printf("\n");
-
-    printf("pwdecode secret \n");
-    for (n=0; n< secretlen; n++) {
-      printf("%.2x ", secret[n]);
-      if ((n % 16) == 15)
-	printf("\n");
-    }
-    printf("\n");
   }
   */
 
@@ -1077,7 +1078,7 @@ int radius_pwdecode(struct radius_t *this,
   }    
 
   /*
-  if (this->debug) {
+  if (_options.debug) {
     printf("pwdecode dest \n");
     for (n=0; n< 32; n++) {
       printf("%.2x ", dst[n]);
@@ -1107,6 +1108,32 @@ int radius_pwencode(struct radius_t *this,
   MD5_CTX context;
   size_t i, n;
 
+  log_dbg("pw encode secret=%s", secret);
+
+  /*
+  if (_options.debug) {
+    char out[1024];
+    out[0]=0;
+
+    log_dbg("pwencode src %s",src);
+    for (n=0; n< srclen; n++) {
+      sprintf(out + strlen(out), "%.2x ", src[n]);
+      if ((n % 16) == 15)
+	sprintf(out+strlen(out),"\n");
+    }
+    log_dbg("%s", out);
+
+    out[0]=0;
+    log_dbg("pwencode authenticator");
+    for (n=0; n< RADIUS_AUTHLEN; n++) {
+      sprintf(out+strlen(out),"%.2x ", authenticator[n]);
+      if ((n % 16) == 15)
+	sprintf(out+strlen(out),"\n");
+    }
+    log_dbg("%s", out);
+  }
+  */
+
   memset(dst, 0, dstsize);
 
   /* Make dstlen multiple of 16 */
@@ -1299,8 +1326,24 @@ void radius_set(struct radius_t *this, unsigned char *hwaddr, int debug) {
   this->debug = debug;
 
   /* Remote radius server parameters */
-  this->hisaddr0.s_addr = _options.radiusserver1.s_addr;
-  this->hisaddr1.s_addr = _options.radiusserver2.s_addr;
+  if (_options.radsec) {
+    inet_aton("127.0.0.1", &this->hisaddr0);
+    this->hisaddr1.s_addr = this->hisaddr0.s_addr;
+
+    this->secretlen = 6;
+    strncpy(this->secret, "radsec", sizeof(this->secret));
+  } else {
+    this->hisaddr0.s_addr = _options.radiusserver1.s_addr;
+    this->hisaddr1.s_addr = _options.radiusserver2.s_addr;
+    
+    if ((this->secretlen = strlen(_options.radiussecret)) > RADIUS_SECRETSIZE) {
+      log_err(0, "Radius secret too long. Truncating to %d characters", 
+	      RADIUS_SECRETSIZE);
+      this->secretlen = RADIUS_SECRETSIZE;
+    }
+    
+    memcpy(this->secret, _options.radiussecret, this->secretlen);
+  }
 
   if (_options.radiusauthport) {
     this->authport = _options.radiusauthport;
@@ -1316,16 +1359,9 @@ void radius_set(struct radius_t *this, unsigned char *hwaddr, int debug) {
     this->acctport = RADIUS_ACCTPORT;
   }
 
-  if ((this->secretlen = strlen(_options.radiussecret)) > RADIUS_SECRETSIZE) {
-    log_err(0, "Radius secret too long. Truncating to %d characters", 
-	    RADIUS_SECRETSIZE);
-    this->secretlen = RADIUS_SECRETSIZE;
-  }
-
-  if (hwaddr)
+  if (hwaddr) {
     memcpy(this->nas_hwaddr, hwaddr, sizeof(this->nas_hwaddr));
-
-  memcpy(this->secret, _options.radiussecret, this->secretlen);
+  }
 
   this->lastreply = 0; /* Start out using server 0 */  
   return;
@@ -1652,6 +1688,7 @@ int radius_decaps(struct radius_t *this, int idx) {
   }
 
   if (!coarequest) {
+
     /* Check that reply is from correct address */
     if ((addr.sin_addr.s_addr != this->hisaddr0.s_addr) &&
 	(addr.sin_addr.s_addr != this->hisaddr1.s_addr)) {
@@ -1795,68 +1832,3 @@ int radius_proxy_ind(struct radius_t *this, int idx) {
   return -1;
 }
 
-struct app_conn_t admin_session;
-
-int chilliauth_radius(struct radius_t *radius) {
-  struct radius_packet_t radius_pack;
-  int ret = -1;
-
-  /*if (admin_session.s_state.authenticated) {
-    terminate_appconn(&admin_session, RADIUS_TERMINATE_CAUSE_SESSION_TIMEOUT);
-    memset(&admin_session.s_state, 0, sizeof(admin_session.s_state));
-    }*/
-  
-  if (radius_default_pack(radius, &radius_pack, RADIUS_CODE_ACCESS_REQUEST)) {
-    log_err(0, "radius_default_pack() failed");
-    return ret;
-  }
-
-  /* adminuser is required */
-  radius_addattr(radius, &radius_pack, RADIUS_ATTR_USER_NAME, 0, 0, 0,
-		 (uint8_t *)_options.adminuser, strlen(_options.adminuser));
-
-  if (_options.adminpasswd)
-    radius_addattr(radius, &radius_pack, RADIUS_ATTR_USER_PASSWORD, 0, 0, 0,
-		   (uint8_t *)_options.adminpasswd, strlen(_options.adminpasswd));
-
-  radius_addattr(radius, &radius_pack, RADIUS_ATTR_SERVICE_TYPE, 0, 0,
-		 RADIUS_SERVICE_TYPE_ADMIN_USER, NULL, 0); 
-  
-  
-  radius_addattr(radius, &radius_pack, RADIUS_ATTR_NAS_PORT_TYPE, 0, 0,
-		 _options.radiusnasporttype, NULL, 0);
-  
-  radius_addnasip(radius, &radius_pack);
-  
-  radius_addcalledstation(radius, &radius_pack);
-
-  if (_options.radiusnasid)
-    radius_addattr(radius, &radius_pack, RADIUS_ATTR_NAS_IDENTIFIER, 0, 0, 0,
-		   (uint8_t *)_options.radiusnasid, strlen(_options.radiusnasid));
-  
-  if (_options.radiuslocationid)
-    radius_addattr(radius, &radius_pack, RADIUS_ATTR_VENDOR_SPECIFIC,
-		   RADIUS_VENDOR_WISPR, RADIUS_ATTR_WISPR_LOCATION_ID, 0,
-		   (uint8_t *)_options.radiuslocationid, strlen(_options.radiuslocationid));
-  
-  if (_options.radiuslocationname)
-    radius_addattr(radius, &radius_pack, RADIUS_ATTR_VENDOR_SPECIFIC,
-		   RADIUS_VENDOR_WISPR, RADIUS_ATTR_WISPR_LOCATION_NAME, 0,
-		   (uint8_t *)_options.radiuslocationname, 
-		   strlen(_options.radiuslocationname));
-
-  radius_addattr(radius, &radius_pack, RADIUS_ATTR_ACCT_SESSION_ID, 0, 0, 0,
-		 (uint8_t*)admin_session.s_state.sessionid, REDIR_SESSIONID_LEN-1);
-
-  if (admin_session.s_state.redir.classlen) {
-    radius_addattr(radius, &radius_pack, RADIUS_ATTR_CLASS, 0, 0, 0,
-		   admin_session.s_state.redir.classbuf,
-		   admin_session.s_state.redir.classlen);
-  }
-
-  radius_addattr(radius, &radius_pack, RADIUS_ATTR_MESSAGE_AUTHENTICATOR, 
-		 0, 0, 0, NULL, RADIUS_MD5LEN);
-
-  return radius_req(radius, &radius_pack, &admin_session); 
-}
-
