@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2009 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -173,16 +173,9 @@
 #define RADIUS_TERMINATE_CAUSE_USER_ERROR           17
 #define RADIUS_TERMINATE_CAUSE_HOST_REQUEST         18
 
-#include "limits.h"
-
-struct radius_packet_t {
-  uint8_t code;
-  uint8_t id;
-  uint16_t length;
-  uint8_t authenticator[RADIUS_AUTHLEN];
-  uint8_t payload[RADIUS_PACKSIZE-RADIUS_HDRSIZE];
-} __attribute__((packed));
-
+#include "radius_pkt.h"
+#include "radius_wispr.h"
+#include "radius_chillispot.h"
 
 struct radius_queue_t {      /* Holder for queued packets */
   int state;                 /* 0=empty, 1=full */
@@ -192,16 +185,8 @@ struct radius_queue_t {      /* Holder for queued packets */
   int lastsent;              /* 0 or 1 indicates last server used */
   struct sockaddr_in peer;   /* Address packet was sent to / received from */
   struct radius_packet_t p;  /* The packet stored */
-  uint16_t seq;              /* The sequence number */
-  uint8_t type;              /* The type of packet */
-  size_t l;                  /* Length of the packet */
-  struct qmsg_t *seqnext;    /* Pointer to next in sequence hash list */
   int next;                  /* Pointer to the next in queue. -1: Last */
   int prev;                  /* Pointer to the previous in queue. -1: First */
-  int this;                  /* Pointer to myself */
-};
-
-struct radius_q {
 };
 
 typedef struct radius_queue_t * radius_queue;
@@ -237,13 +222,8 @@ struct radius_t {
   int first;                     /* First packet in queue (oldest timeout) */
   int last;                      /* Last packet in queue (youngest timeout) */
 
-  int listsize;                  /* Total number of addresses */
-  int hashsize;                  /* Size of hash table */
-  int hashlog;                   /* Log2 size of hash table */
-  int hashmask;                  /* Bitmask for calculating hash */
-
-  int (*cb_ind)  (struct radius_t *radius, struct radius_packet_t *pack,
-		  struct sockaddr_in *peer);
+  int (*cb_ind)       (struct radius_t *radius, struct radius_packet_t *pack,
+		       struct sockaddr_in *peer);
   int (*cb_auth_conf) (struct radius_t *radius, struct radius_packet_t *pack,
 		       struct radius_packet_t *pack_req, void *cbp);
   int (*cb_acct_conf) (struct radius_t *radius, struct radius_packet_t *pack,
@@ -252,37 +232,6 @@ struct radius_t {
 		       struct sockaddr_in *peer);
 };
 
-struct radiusm_t {
-  struct in_addr addr;           /* IP address of this member */
-  int inuse;                     /* 0=available; 1= inuse */
-  struct RADIUSm_t *nexthash;    /* Linked list part of hash table */
-  struct RADIUSm_t *prev, *next; /* Double linked list of available members */
-  struct RADIUS_t *parent;       /* Pointer to parent */
-  void *peer;                    /* Pointer to peer protocol handler */
-};
-
-
-struct radius_vsattr_t {
-  uint32_t i;  /* vendor-id */
-  uint8_t t;   /* vsa type */
-  uint8_t l;   /* length */
-  union {
-    uint32_t i;
-    uint8_t  t[RADIUS_ATTR_VLEN-4];
-  } v;         /* value */
-} __attribute__((packed));
-
-
-struct radius_attr_t {
-  uint8_t t;   /* type */
-  uint8_t l;   /* length */
-  union {
-    uint32_t i;
-    uint8_t  t[RADIUS_ATTR_VLEN];
-    struct radius_vsattr_t vv;
-  } v;        /* value */
-} __attribute__((packed));
-
 
 /* Create new radius instance */
 int radius_new(struct radius_t **this, 
@@ -400,6 +349,4 @@ int radius_authresp_authenticator(struct radius_t *this,
 int radius_hmac_md5(struct radius_t *this, struct radius_packet_t *pack, 
 		    char *secret, int secretlen, uint8_t *dst);
 
-int chilliauth_radius(struct radius_t *radius);
-
 #endif	/* !_RADIUS_H */
