@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2009 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,17 +17,7 @@
  * 
  */
 
-#include "system.h"
-#include "syserr.h"
-#include "radius.h"
-#include "radius_wispr.h"
-#include "radius_chillispot.h"
-#include "redir.h"
-#include "md5.h"
-#include "dhcp.h"
 #include "chilli.h"
-#include "options.h"
-#include "ssl.h"
 
 static int optionsdebug = 0; /* TODO: Should be changed to instance */
 
@@ -45,11 +35,11 @@ static int redir_getparam(struct redir_t *redir, char *src, char *param, bstring
 static uint8_t radius_packet_id = 0;
 extern time_t mainclock;
 
-/* Termination handler for clean shutdown */
+/* Termination handler for clean shutdown 
 static void redir_termination(int signum) {
   log_dbg("Terminating redir client!");
   exit(0);
-}
+}*/
 
 /* Alarm handler for ensured shutdown */
 static void redir_alarm(int signum) {
@@ -149,7 +139,7 @@ static int redir_xmlencode(char *src, int srclen, char *dst, int dstsize) {
 */
 
 static void redir_http(bstring s, char *code) {
-  bassigncstr(s, "HTTP/1.1 ");
+  bassigncstr(s, "HTTP/1.0 ");
   bcatcstr(s, code);
   bcatcstr(s, "\r\n");
   bcatcstr(s, "Connection: close\r\nCache-Control: no-cache, must-revalidate\r\n");
@@ -334,10 +324,10 @@ static int bstring_buildurl(bstring str, struct redir_conn_t *conn,
     bconcat(str, bt2);
   }
 
-  if (redir->radiusnasid) {
+  if (_options.radiusnasid) {
     bcatcstr(str, amp);
     bcatcstr(str, "nasid=");
-    bassigncstr(bt, redir->radiusnasid);
+    bassigncstr(bt, _options.radiusnasid);
     redir_urlencode(bt, bt2);
     bconcat(str, bt2);
   }
@@ -358,6 +348,39 @@ static int bstring_buildurl(bstring str, struct redir_conn_t *conn,
     bconcat(str, bt2);
   }
 
+#ifdef ENABLE_PROXYVSA
+  if (_options.proxy_loc[0].attr) {
+    
+    log_dbg("vsalen %d", conn->s_state.redir.vsalen);
+    
+    if (conn->s_state.redir.vsalen) {
+      uint16_t len = conn->s_state.redir.vsalen;
+      uint16_t offset = 0;
+      struct radius_attr_t *t;
+
+      while (offset < len) {
+	t = (struct radius_attr_t *)(conn->s_state.redir.vsa + offset);
+	offset += t->l;
+	if (t->t == RADIUS_ATTR_VENDOR_SPECIFIC && 
+	    ntohl(t->v.vv.i) == RADIUS_VENDOR_CHILLISPOT &&
+	    t->v.vv.t ==  RADIUS_ATTR_CHILLISPOT_LOCATION) {
+
+	  bcatcstr(str, amp);
+	  bcatcstr(str, "loc=");
+	  bassignblk(bt, (void *) t->v.vv.v.t, t->v.vv.l - 2);
+	  redir_urlencode(bt, bt2);
+
+	  log_dbg("found %.*s",
+		  bt->slen, bt->data);
+	  
+	  bconcat(str, bt2);
+	  break;
+	}
+      }
+    }
+  }
+#endif
+
   if (conn->lang[0]) {
     bcatcstr(str, amp);
     bcatcstr(str, "lang=");
@@ -382,9 +405,16 @@ static int bstring_buildurl(bstring str, struct redir_conn_t *conn,
      */
     bcatcstr(str, amp);
     bcatcstr(str, "ssl=");
-    bassignformat(bt, "https://%s:%d/", 
-		  inet_ntoa(_options.uamalias),
-		  _options.uamuiport);
+    if (_options.uamaliasname && _options.domain) {
+      bassignformat(bt, "https://%s.%s:%d/", 
+                    _options.uamaliasname,
+                    _options.domain,
+                    _options.uamuiport);
+    } else {
+      bassignformat(bt, "https://%s:%d/", 
+                    inet_ntoa(_options.uamalias),
+                    _options.uamuiport);
+    }
     redir_urlencode(bt, bt2);
     bconcat(str, bt2);
   }
@@ -443,8 +473,7 @@ static int redir_xmlreply(struct redir_t *redir,
 			  char* reply, char* redirurl, bstring b) {
   bstring bt;
 
-  if (redir->no_uamwispr && 
-      !(redir->chillixml)) return 0;
+  if (_options.no_uamwispr && !(_options.chillixml)) return 0;
 
   bt = bfromcstr("");
 
@@ -452,7 +481,7 @@ static int redir_xmlreply(struct redir_t *redir,
 	   "<!--\r\n"
 	   "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n");
   
-  if (!redir->no_uamwispr) {
+  if (!_options.no_uamwispr) {
     bcatcstr(b, 
 	     "<WISPAccessGatewayParam\r\n"
 	     "  xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\r\n"
@@ -551,13 +580,15 @@ static int redir_xmlreply(struct redir_t *redir,
 	       "<Redirect>\r\n"
 	       "<AccessProcedure>1.0</AccessProcedure>\r\n");
 
-      if (redir->radiuslocationid) {
-	bassignformat(bt, "<AccessLocation>%s</AccessLocation>\r\n", redir->radiuslocationid);
+      if (_options.radiuslocationid) {
+	bassignformat(bt, "<AccessLocation>%s</AccessLocation>\r\n", 
+		      _options.radiuslocationid);
 	bconcat(b, bt);
       }
 
-      if (redir->radiuslocationname) {
-	bassignformat(bt, "<LocationName>%s</LocationName>\r\n", redir->radiuslocationname);
+      if (_options.radiuslocationname) {
+	bassignformat(bt, "<LocationName>%s</LocationName>\r\n", 
+		      _options.radiuslocationname);
 	bconcat(b, bt);
       }
 
@@ -625,7 +656,7 @@ static int redir_xmlreply(struct redir_t *redir,
   }
 
 #ifdef ENABLE_CHILLIXML
-  if (redir->chillixml) {
+  if (_options.chillixml) {
     bcatcstr(b, "<ChilliSpotSession>\r\n");
     switch (res) {
     case REDIR_SPLASH:
@@ -875,10 +906,10 @@ static int redir_json_reply(struct redir_t *redir, int res, struct redir_conn_t
 
   if (flg & FLG_loc) {
     bcatcstr(json,",\"location\":{\"name\":\"");
-    if (redir->locationname)
-      bcatcstr(json, redir->locationname);
-    else if (redir->radiuslocationname)
-      bcatcstr(json, redir->radiuslocationname);
+    if (_options.locationname)
+      bcatcstr(json, _options.locationname);
+    else if (_options.radiuslocationname)
+      bcatcstr(json, _options.radiuslocationname);
     bcatcstr(json,"\"");
     bcatcstr(json,"}");
   }
@@ -886,7 +917,7 @@ static int redir_json_reply(struct redir_t *redir, int res, struct redir_conn_t
   if (flg & FLG_redir) {
     bassignformat(tmp , "http://%s:%d/logoff", 
 		  inet_ntoa(redir->addr), redir->port);
-
+    
     session_redir_json_fmt(json, userurl, redirurl, tmp, hismac);
   }
 
@@ -1167,9 +1198,15 @@ int redir_ipc(struct redir_t *redir) {
     char *statedir = _options.statedir ? _options.statedir : DEFSTATEDIR;
     char filedest[512];
     
-    snprintf(filedest, sizeof(filedest), "%s/%s", statedir, 
-	     _options.unixipc ? _options.unixipc : "chilli.ipc");
-    
+    if (!_options.unixipc) {
+      snprintf(filedest, sizeof(filedest), "%s/chilli.ipc", statedir);
+    } else {
+      if (_options.unixipc[0]=='/')
+        snprintf(filedest, sizeof(filedest), "%s", _options.unixipc);
+      else
+        snprintf(filedest, sizeof(filedest), "%s/%s", statedir, _options.unixipc);
+    }
+
     local.sun_family = AF_UNIX;
     
     strcpy(local.sun_path, filedest);
@@ -1177,7 +1214,7 @@ int redir_ipc(struct redir_t *redir) {
     
     if (bind(sock, (struct sockaddr *)&local, 
 	     sizeof(struct sockaddr_un)) == -1) {
-      log_err(errno, "could bind UNIX Socket!");
+      log_err(errno, "could bind UNIX Socket to %s!", filedest);
       close(sock);
       sock = -1;
     } else {
@@ -1187,6 +1224,12 @@ int redir_ipc(struct redir_t *redir) {
 	sock = -1;
       } else {
 	redir->msgfd = sock;
+
+	if (_options.uid) {
+	  if (chown(filedest, _options.uid, _options.gid)) {
+	    log_err(errno, "could not chown() %s", filedest);
+	  }
+	}
       }
     }
   }
@@ -1244,8 +1287,6 @@ void redir_set(struct redir_t *redir, uint8_t *hwaddr, int debug) {
   optionsdebug = debug; /* TODO: Do not change static variable from instance */
   redir->debug = debug;
 
-  redir->no_uamwispr = _options.no_uamwispr;
-  redir->chillixml = _options.chillixml;
   redir->url = _options.uamurl;
   redir->homepage = _options.uamhomepage;
   redir->secret = _options.uamsecret;
@@ -1253,16 +1294,6 @@ void redir_set(struct redir_t *redir, uint8_t *hwaddr, int debug) {
   redir->vlan = _options.vlan;
   redir->nasmac = _options.nasmac;
   redir->nasip = _options.nasip;
-  redir->radiusserver0 = _options.radiusserver1;
-  redir->radiusserver1 = _options.radiusserver2;
-  redir->radiusauthport = _options.radiusauthport;
-  redir->radiusacctport = _options.radiusacctport;
-  redir->radiussecret  = _options.radiussecret;
-  redir->radiusnasid  = _options.radiusnasid;
-  redir->radiuslocationid  = _options.radiuslocationid;
-  redir->radiuslocationname  = _options.radiuslocationname;
-  redir->locationname  = _options.locationname;
-  redir->radiusnasporttype = _options.radiusnasporttype;
 
   if (hwaddr) {
     memcpy(redir->nas_hwaddr, hwaddr, sizeof(redir->nas_hwaddr));
@@ -1307,7 +1338,8 @@ static int redir_getparam(struct redir_t *redir, char *src, char *param, bstring
 
 /* Read the an HTTP request from a client */
 static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
-			struct redir_conn_t *conn, struct redir_httpreq_t *httpreq) {
+			struct redir_conn_t *conn, struct redir_httpreq_t *httpreq,
+			int forked) {
   int fd = sock->fd[0];
   fd_set fds;
   struct timeval idleTime;
@@ -1315,7 +1347,7 @@ static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
   ssize_t recvlen = 0;
   size_t buflen = 0;
   char buffer[REDIR_MAXBUFFER];
-  int i, lines=0, done=0;
+  int i, lines=0, done=0, eoh=0;
   char *eol;
 
   char read_waiting;
@@ -1335,7 +1367,7 @@ static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
     }
 #endif
     
-    if (!read_waiting) {
+    if (!read_waiting && forked) {
 
       /*
        *  If not already with data from the SSL layer, wait for it. 
@@ -1372,7 +1404,7 @@ static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
       }
     }
 
-    if (read_waiting) {
+    if (read_waiting || !forked) {
 
       if (buflen + 2 >= sizeof(buffer)) { /* ensure space for a least one more byte + null */
         log_err(0, "Too much data in http request!");
@@ -1391,15 +1423,20 @@ static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
       recvlen = recv(fd, buffer + buflen, httpreq->allow_post ? 1 : sizeof(buffer) - 1 - buflen, 0);
 
       if (recvlen < 0) {
-	if (errno != ECONNRESET)
-	  log_err(errno, "%s_read() failed!", 
+	if (errno == EWOULDBLOCK && !forked) {
+	  log_dbg("Continue... (would block)");
+	  recvlen = 0;
+	}
+	else if (errno != ECONNRESET) {
+	  log_err(errno, "%s_read(%d) failed!", 
 #ifdef HAVE_SSL
 		  sock->sslcon ? "SSL" : 
 #endif
-		  "redir");
-	return -1;
+		  "redir", fd);
+	  return -1;
+	}
       }
-
+      
       if (recvlen == 0) done=1;
       else if (httpreq->data_in) {
 	bcatblk(httpreq->data_in, buffer + buflen, recvlen);
@@ -1408,21 +1445,38 @@ static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
       buflen += recvlen;
       buffer[buflen] = 0;
     }
-
+    
+    if (httpreq->data_in && forked) {
+      log_err(0,"should not happen");
+      exit(1);
+    }
+    
+    if (httpreq->data_in && !forked) {
+      if (httpreq->data_in->slen >= sizeof(buffer)) {
+	log_err(0, "buffer too long (%d)", httpreq->data_in->slen);
+	return -1;
+      } else {
+	buflen = httpreq->data_in->slen;
+	memcpy(buffer, httpreq->data_in->data, buflen);
+	buffer[buflen] = 0;
+      }
+    }
+    
     if (buflen == 0) {
       log_dbg("No data in HTTP request!");
+      if (!forked) return 1;
       return -1;
     }
 
     while ((eol = strstr(buffer, "\r\n"))) {
       size_t linelen = eol - buffer;
       *eol = 0;
-
+      
       if (lines++ == 0) { /* first line */
 	size_t dstlen = 0;
 	char *p1 = buffer;
 	char *p2;
-
+	
 	if      (!strncmp("GET ",  p1, 4)) { p1 += 4; }
 	else if (!strncmp("HEAD ", p1, 5)) { p1 += 5; }
 	else if (httpreq->allow_post && !strncmp("POST ", p1, 5)) { 
@@ -1435,12 +1489,12 @@ static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
 
 	while (*p1 == ' ') p1++; /* Advance through additional white space */
 	if (*p1 == '/') p1++;
-	else return -1;
+	else { log_err(0, "parse error"); return -1; }
 	
 	/* The path ends with a ? or a space */
 	p2 = strchr(p1, '?');
 	if (!p2) p2 = strchr(p1, ' ');
-	if (!p2) return -1;
+	if (!p2) { log_err(0, "parse error"); return -1; }
 	dstlen = p2 - p1;
 
 	if (dstlen >= sizeof(httpreq->path)-1) 
@@ -1503,6 +1557,7 @@ static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
 	/* end of headers */
 	log_dbg("end of http-request");
 	done = 1;
+	eoh = 1;
 	break;
       } else { 
 	/* headers */
@@ -1543,7 +1598,7 @@ static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
 	  if (len >= sizeof(conn->httpcookie)-1)
 	    len = sizeof(conn->httpcookie)-1;
 	  strncpy(conn->httpcookie, p, len);
-	  conn->httpcookie[len]=0;
+	  conn->httpcookie[len] = 0;
 	  log_dbg("Cookie: %s",conn->httpcookie);
 	}
       }
@@ -1551,10 +1606,16 @@ static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
       /* shift buffer */
       linelen += 2;
       for (i = 0; i < (int)(buflen - linelen); i++)
-	buffer[i] = buffer[(int)linelen+i];
+	buffer[i] = buffer[(int)linelen + i];
 
       buflen -= linelen;
     }
+
+    if (!forked && !eoh) {
+      log_dbg("Didn't see end of headers, continue...");
+      log_dbg("%.*s",httpreq->data_in->slen,httpreq->data_in->data);
+      return 1;
+    }
   }
 
   switch(conn->type) {
@@ -1601,10 +1662,10 @@ static int redir_getreq(struct redir_t *redir, struct redir_socket_t *sock,
       }
       else if (!redir_getparam(redir, httpreq->qs, "password", bt)) {
 	conn->password_len = bt->slen / 2;
-	log_dbg("Password length: %d",conn->password_len);
 	if (conn->password_len > RADIUS_PWSIZE)
 	  conn->password_len = RADIUS_PWSIZE;
 	redir_hextochar(bt->data, bt->slen, conn->password, conn->password_len);
+	log_dbg("Password(%d): %.*s",conn->password_len,conn->password_len,conn->password);
 	conn->chap = 0;
 	conn->chappassword[0] = 0;
       } else {
@@ -1762,7 +1823,6 @@ static int redir_radius(struct redir_t *redir, struct in_addr *addr,
 
   MD5_CTX context;
 
-  char mac[REDIR_MACSTRLEN+1];
   char url[REDIR_URL_LEN];
   int n, m;
 
@@ -1784,24 +1844,17 @@ static int redir_radius(struct redir_t *redir, struct in_addr *addr,
   radius_set_cb_auth_conf(radius, redir_cb_radius_auth_conf);
 
   radius_default_pack(radius, &radius_pack, RADIUS_CODE_ACCESS_REQUEST);
-  
+
   log_dbg("created radius packet (code=%d, id=%d, len=%d)\n",
 	  radius_pack.code, radius_pack.id, ntohs(radius_pack.length));
   
-  radius_addattr(radius, &radius_pack, RADIUS_ATTR_USER_NAME, 0, 0, 0,
-		 (uint8_t*) conn->s_state.redir.username, strlen(conn->s_state.redir.username));
-
-  /* If lang on logon url, then send it with attribute ChilliSpot-Lang */
   if(conn->lang[0]) 
     radius_addattr(radius, &radius_pack, RADIUS_ATTR_VENDOR_SPECIFIC, 
 		   RADIUS_VENDOR_CHILLISPOT, RADIUS_ATTR_CHILLISPOT_LANG, 
 		   0, (uint8_t*) conn->lang, strlen(conn->lang));
 
-  if (_options.radiusoriginalurl)
-    radius_addattr(radius, &radius_pack, RADIUS_ATTR_VENDOR_SPECIFIC, 
-		   RADIUS_VENDOR_CHILLISPOT, RADIUS_ATTR_CHILLISPOT_ORIGINALURL, 
-		   0, (uint8_t*) conn->s_state.redir.userurl, strlen(conn->s_state.redir.userurl));
-
+  radius_addattr(radius, &radius_pack, RADIUS_ATTR_USER_NAME, 0, 0, 0,
+		 (uint8_t*) conn->s_state.redir.username, strlen(conn->s_state.redir.username));
 
   if (redir->secret && *redir->secret) {
     /* fprintf(stderr,"SECRET: [%s]\n",redir->secret); */
@@ -1838,7 +1891,8 @@ static int redir_radius(struct redir_t *redir, struct in_addr *addr,
 	redir_challenge(peer_challenge);*/
       
       GenerateNTResponse(chap_challenge, /*peer*/chap_challenge,
-			 conn->s_state.redir.username, strlen(conn->s_state.redir.username),
+			 (u_char *)conn->s_state.redir.username, 
+			 strlen(conn->s_state.redir.username),
 			 user_password, conn->password_len,
 			 ntresponse);
       
@@ -1883,47 +1937,23 @@ static int redir_radius(struct redir_t *redir, struct in_addr *addr,
     /* peer challenge - same as auth challenge */
     memcpy(response + 2, chap_challenge, 16); 
     memcpy(response + 26, conn->chappassword, 24);
-
+    
     radius_addattr(radius, &radius_pack, 
 		   RADIUS_ATTR_VENDOR_SPECIFIC,
 		   RADIUS_VENDOR_MS, RADIUS_ATTR_MS_CHAP_CHALLENGE, 0,
 		   chap_challenge, 16);
-
+    
     radius_addattr(radius, &radius_pack, 
 		   RADIUS_ATTR_VENDOR_SPECIFIC,
 		   RADIUS_VENDOR_MS, RADIUS_ATTR_MS_CHAP2_RESPONSE, 0,
 		   response, 50);
   }
 
-  radius_addnasip(radius, &radius_pack);
-
-  radius_addattr(radius, &radius_pack, RADIUS_ATTR_SERVICE_TYPE, 0, 0,
-		 _options.framedservice ? RADIUS_SERVICE_TYPE_FRAMED :
-		 RADIUS_SERVICE_TYPE_LOGIN, NULL, 0); /* WISPr_V1.0 */
-
-  radius_addattr(radius, &radius_pack, RADIUS_ATTR_FRAMED_IP_ADDRESS, 0, 0,
-		 ntohl(conn->hisip.s_addr), NULL, 0); /* WISPr_V1.0 */
-
-  /* Include his MAC address */
-  snprintf(mac, REDIR_MACSTRLEN+1, "%.2X-%.2X-%.2X-%.2X-%.2X-%.2X",
-	   conn->hismac[0], conn->hismac[1],
-	   conn->hismac[2], conn->hismac[3],
-	   conn->hismac[4], conn->hismac[5]);
-  
-  radius_addattr(radius, &radius_pack, RADIUS_ATTR_CALLING_STATION_ID, 0, 0, 0,
-		 (uint8_t*) mac, REDIR_MACSTRLEN);
-
-  radius_addcalledstation(radius, &radius_pack);
-
-
-  if (redir->radiusnasid)
-    radius_addattr(radius, &radius_pack, RADIUS_ATTR_NAS_IDENTIFIER, 0, 0, 0,
-		   (uint8_t*) redir->radiusnasid, 
-		   strlen(redir->radiusnasid)); /* WISPr_V1.0 */
-
-
-  radius_addattr(radius, &radius_pack, RADIUS_ATTR_ACCT_SESSION_ID, 0, 0, 0,
-		 (uint8_t*) conn->s_state.sessionid, REDIR_SESSIONID_LEN-1);
+  chilli_req_attrs(radius, &radius_pack, 
+		   _options.framedservice ? RADIUS_SERVICE_TYPE_FRAMED : 
+		   RADIUS_SERVICE_TYPE_LOGIN,
+		   conn->nasport, conn->hismac,
+		   &conn->hisip, &conn->s_state);
 
   if (conn->s_state.redir.classlen) {
     radius_addattr(radius, &radius_pack, RADIUS_ATTR_CLASS, 0, 0, 0,
@@ -1931,24 +1961,6 @@ static int redir_radius(struct redir_t *redir, struct in_addr *addr,
 		   conn->s_state.redir.classlen);
   }
 
-  radius_addattr(radius, &radius_pack, RADIUS_ATTR_NAS_PORT_TYPE, 0, 0,
-		 redir->radiusnasporttype, NULL, 0);
-
-  radius_addattr(radius, &radius_pack, RADIUS_ATTR_NAS_PORT, 0, 0,
-		 conn->nasport, NULL, 0);
-  
-  if (redir->radiuslocationid)
-    radius_addattr(radius, &radius_pack, RADIUS_ATTR_VENDOR_SPECIFIC,
-		   RADIUS_VENDOR_WISPR, RADIUS_ATTR_WISPR_LOCATION_ID, 0,
-		   (uint8_t*) redir->radiuslocationid,
-		   strlen(redir->radiuslocationid));
-
-  if (redir->radiuslocationname)
-    radius_addattr(radius, &radius_pack, RADIUS_ATTR_VENDOR_SPECIFIC,
-		   RADIUS_VENDOR_WISPR, RADIUS_ATTR_WISPR_LOCATION_NAME, 0,
-		   (uint8_t*) redir->radiuslocationname, 
-		   strlen(redir->radiuslocationname));
-
   if (snprintf(url, sizeof(url)-1, "http://%s:%d/logoff", 
 	       inet_ntoa(redir->addr), redir->port) > 0)
     radius_addattr(radius, &radius_pack, RADIUS_ATTR_VENDOR_SPECIFIC,
@@ -1960,13 +1972,6 @@ static int redir_radius(struct redir_t *redir, struct in_addr *addr,
 		   RADIUS_VENDOR_CHILLISPOT, RADIUS_ATTR_CHILLISPOT_CONFIG, 
 		   0, (uint8_t*)"allow-openidauth", 16);
 
-#ifdef ENABLE_IEEE8021Q
-  if (conn->s_state.tag8021q)
-    radius_addattr(radius, &radius_pack, RADIUS_ATTR_VENDOR_SPECIFIC,
-		   RADIUS_VENDOR_CHILLISPOT, RADIUS_ATTR_CHILLISPOT_VLAN_ID, 
-		   (uint32_t)(ntohl(conn->s_state.tag8021q) & 0x0FFF), 0, 0);
-#endif
-
   radius_addattr(radius, &radius_pack, RADIUS_ATTR_MESSAGE_AUTHENTICATOR, 
 		 0, 0, 0, NULL, RADIUS_MD5LEN);
 
@@ -2231,7 +2236,7 @@ int redir_accept(struct redir_t *redir, int idx) {
 
   } else {
 
-    return redir_main(redir, 0, 1, &address, &baddress, idx, 1);
+    return redir_main(redir, 0, 1, &address, &baddress, idx, 0);
 
   }
 
@@ -2349,7 +2354,8 @@ pid_t redir_fork(int in, int out) {
 
 int redir_main_exit(struct redir_t *redir, struct redir_httpreq_t *httpreq, 
 		    struct redir_socket_t *socket, int forked) {
-  if (httpreq->data_in) bdestroy(httpreq->data_in);
+  /*if (httpreq->data_in) bdestroy(httpreq->data_in);*/
+  if (!forked) return 0; /*XXXX*/
 #ifdef HAVE_SSL
   if (socket->sslcon) {
     openssl_shutdown(socket->sslcon, 2);
@@ -2361,21 +2367,24 @@ int redir_main_exit(struct redir_t *redir, struct redir_httpreq_t *httpreq,
   return _redir_close(socket->fd[0], socket->fd[1]);
 }
 
-static int redir_signal2(int sig) {
+#ifdef HAVE_SSL
+static void redir_signal2(int sig) {
   log_dbg("signal(%d)=%d", getpid(),sig);
 }
 static int redir_close_on_signal[2]={0,0};
-static int redir_signal(int sig) {
+static void redir_signal(int sig) {
   close(redir_close_on_signal[0]);
   close(redir_close_on_signal[1]);
   log_dbg("signal(%d)=%d", getpid(),sig);
 }
+#endif
 
 int redir_main(struct redir_t *redir, 
 	       int infd, int outfd, 
 	       struct sockaddr_in *address, 
 	       struct sockaddr_in *baddress,
-	       int isui, int forked) {
+	       int isui, redir_request *rreq) {
+
   char hexchal[1+(2*REDIR_MD5LEN)];
   unsigned char challenge[REDIR_MD5LEN];
   size_t bufsize = REDIR_MAXBUFFER;
@@ -2399,13 +2408,17 @@ int redir_main(struct redir_t *redir,
   struct redir_socket_t socket;
   struct redir_httpreq_t httpreq;
 
+  /* We are forked when the redir_request is null */
+  int forked = (rreq == 0);
+  int err;
+
   memset(&httpreq,0,sizeof(httpreq));
   httpreq.allow_post = isui || _options.uamallowpost;
 
   mainclock_tick();
 
-  if (!forked) {
-    httpreq.data_in = bfromcstr("");
+  if (rreq) {
+    httpreq.data_in = rreq->wbuf;
   }
 
 #define redir_memcopy(msgtype) \
@@ -2489,16 +2502,52 @@ int redir_main(struct redir_t *redir,
 
 #ifdef HAVE_SSL
   if ((conn.flags & USING_SSL) == USING_SSL) {
-    socket.sslcon = openssl_accept_fd(initssl(), socket.fd[0], 30);
+    char done = 0, loop = 0;
+
+    if (!rreq || !rreq->sslcon) {
+      socket.sslcon = openssl_accept_fd(initssl(), socket.fd[0], 30);
+      if (rreq) {
+	/* we were forked */
+	rreq->sslcon = socket.sslcon;
+      } else {
+	/* not forked, so loop */
+	loop = 1;
+      }
+    } else {
+      socket.sslcon = rreq->sslcon;
+    }
+
+    while (!done) {
+      switch(openssl_check_accept(socket.sslcon)) {
+      case -1:
+	return redir_main_exit(redir, &httpreq, &socket, forked);
+      case 1:
+	if (!loop) {
+	  log_dbg("Continue... SSL pending");
+	  return 1;
+	}
+      case 0: done = 1;
+      default: break;
+      }
+      if (!loop) done = 1;
+    }
   }
 #endif
 
+
   termstate = REDIR_TERM_GETREQ;
-  if (redir_getreq(redir, &socket, &conn, &httpreq)) {
-    log_dbg("Error calling get_req. Terminating\n");
+  switch (err = redir_getreq(redir, &socket, &conn, &httpreq, forked)) {
+  case 0: 
+    break;
+  case 1: 
+    log_dbg("Continue...");
+    return 1;
+  default:
+    log_dbg("Error calling get_req. Terminating %d", err);
     return redir_main_exit(redir, &httpreq, &socket, forked);
   }
 
+
   log_dbg("Processing HTTP%s Request", (conn.flags & USING_SSL) ? "S" : "");
   
   if (conn.type == REDIR_WWW) {
@@ -2552,18 +2601,18 @@ int redir_main(struct redir_t *redir,
 	  return redir_main_exit(redir, &httpreq, &socket, forked);
 	}
       }
-
+      
       if (parse) {
-
+	
 	if (!_options.wwwbin) {
 	  log_err(0, "the 'wwwbin' setting must be configured for CGI use");
 	  return redir_main_exit(redir, &httpreq, &socket, forked);
 	}
-
+	
 	if (clear_nonblocking(socket.fd[0])) {
 	  log_err(errno, "fcntl() failed");
 	}
-
+	
 #ifdef HAVE_SSL
 	if (socket.sslcon) {
 
@@ -2684,11 +2733,11 @@ int redir_main(struct redir_t *redir,
 	  setenv("CHI_SESSION_ID", conn.s_state.sessionid, 1);
 	  setenv("CHI_USERNAME", conn.s_state.redir.username, 1);
 	  setenv("CHI_USERURL", conn.s_state.redir.userurl, 1);
-	  sprintf(buffer, "%ld", conn.s_state.input_octets);
+	  sprintf(buffer, "%lld", conn.s_state.input_octets);
 	  setenv("CHI_INPUT_BYTES", buffer, 1);
-	  sprintf(buffer, "%ld", conn.s_state.output_octets);
+	  sprintf(buffer, "%lld", conn.s_state.output_octets);
 	  setenv("CHI_OUTPUT_BYTES", buffer, 1);
-	  sprintf(buffer, "%ld", conn.s_params.sessiontimeout);
+	  sprintf(buffer, "%lld", conn.s_params.sessiontimeout);
 	  setenv("CHI_SESSION_TIMEOUT", buffer, 1);
 	  
 	  redir_chartohex(conn.s_state.redir.uamchal, buffer, REDIR_MD5LEN);
@@ -2703,7 +2752,8 @@ int redir_main(struct redir_t *redir,
 	return redir_main_exit(redir, &httpreq, &socket, forked);
       }
 
-      if (!chroot(_options.wwwdir) && !chdir("/")) {
+      if ( (_options.uid == 0 && !chroot(_options.wwwdir) && !chdir("/")) ||
+	   (_options.uid != 0 && !chdir(_options.wwwdir)) ) {
 	
 	fd = open(filename, O_RDONLY);
 	
@@ -2714,7 +2764,7 @@ int redir_main(struct redir_t *redir,
 	  }
 	  
 	  buflen = snprintf(buffer, bufsize,
-			    "HTTP/1.1 200 OK\r\n"
+			    "HTTP/1.0 200 OK\r\n"
 			    "Connection: close\r\n"
 			    "Content-type: %s\r\n\r\n", ctype);
 	  
@@ -2730,7 +2780,7 @@ int redir_main(struct redir_t *redir,
 	} 
 	else log_err(0, "could not open local content file %s!", filename);
       }
-      else log_err(0, "chroot to %s was not successful\n", _options.wwwdir); 
+      else log_err(0, "chroot/chdir to %s was not successful\n", _options.wwwdir); 
 
       return _redir_close_exit(infd, outfd); /* which exits */
     }
@@ -2785,6 +2835,18 @@ int redir_main(struct redir_t *redir,
        conn.response = REDIR_SUCCESS;
     }
     else {
+
+      if (!forked) {
+	/*
+	 *  When waiting for RADIUS, we need to be forked.
+	 *  TODO: make redir_radius asynchronous.
+	 */
+	pid_t forkpid = redir_fork(infd, outfd);
+	if (forkpid) { /* parent or error */
+	  return redir_main_exit(redir, &httpreq, &socket, forked);
+	}
+      }
+
       termstate = REDIR_TERM_RADIUS;
 
       if (optionsdebug) 
@@ -2929,7 +2991,7 @@ int redir_main(struct redir_t *redir,
     }
 
   case REDIR_MSDOWNLOAD:
-    buflen = snprintf(buffer, bufsize, "HTTP/1.1 403 Forbidden\r\n\r\n");
+    buflen = snprintf(buffer, bufsize, "HTTP/1.0 403 Forbidden\r\n\r\n");
     tcp_write(&socket, buffer, buflen);
     return redir_main_exit(redir, &httpreq, &socket, forked);
 
@@ -2943,12 +3005,15 @@ int redir_main(struct redir_t *redir,
     log_dbg("redir_accept: Original request");
 
 
+  /*
+   *  XXX: chilli_redir
+   */
   if (redir->cb_handle_url) {
-    switch (redir->cb_handle_url(redir, &conn, &httpreq, &socket, redir->cb_handle_url_ctx)) {
+    switch (redir->cb_handle_url(redir, &conn, &httpreq, &socket, address, rreq)) {
     case -1: 
-      return redir_main_exit(redir, &httpreq, &socket, forked);
+      return -1;
     case 0: 
-      return 0;
+      return 1;
     default:
       break;
     }
@@ -2963,23 +3028,20 @@ int redir_main(struct redir_t *redir,
   }
   else {
     redir_chartohex(conn.s_state.redir.uamchal, hexchal, REDIR_MD5LEN);
-    /*
-	redir_memcopy(REDIR_CHALLENGE);
-	redir_msg_send(REDIR_MSG_OPT_REDIR);
-    */
+    msg.mtype = splash ? REDIR_ALREADY : REDIR_NOTYET;
+    redir_msg_send(REDIR_MSG_OPT_REDIR);
   }
 
   log_dbg("---->>> challenge: %s", hexchal);
 
-  if (_options.macreauth && 
-      !conn.s_state.authenticated) {
+  if (_options.macreauth && !conn.s_state.authenticated) {
     msg.mtype = REDIR_MACREAUTH;
     redir_msg_send(0);
   }
-
+  
   if (redir->homepage ||
       ((conn.s_params.flags & REQUIRE_REDIRECT) && conn.s_params.url[0])) {
-
+    
     char * base_url = (conn.s_params.flags & REQUIRE_REDIRECT && 
 		       conn.s_params.url[0]) ? (char *)conn.s_params.url : redir->homepage;
 
