@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2009 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -122,6 +122,38 @@ struct redir_httpreq_t {
   size_t clen;
 };
 
+typedef struct _redir_request {
+  int index;
+
+  int uiidx;
+
+  char inuse:1;
+  char proxy:1;
+  
+  bstring url;
+  bstring data;
+  bstring post;
+  bstring wbuf;
+
+  time_t last_active;
+
+  struct sockaddr_in baddr;
+  
+  struct conn_t conn;
+  
+#ifdef HAVE_SSL
+  openssl_con *sslcon;
+#endif
+  int socket_fd;
+
+#define REDIR_SOCKET_FD (1<<0)
+#define REDIR_CONN_FD   (1<<1)
+  char state;
+
+  struct _redir_request *prev, *next;
+
+} redir_request;
+
 struct redir_socket_t {
   int fd[2];
 #ifdef HAVE_SSL
@@ -151,18 +183,7 @@ struct redir_t {
   char *nasmac;
   char *nasip;
   struct in_addr radiuslisten;
-  struct in_addr radiusserver0;
-  struct in_addr radiusserver1;
-  uint16_t radiusauthport;
-  uint16_t radiusacctport;
-  char *radiussecret;
-  char *radiusnasid;
-  char* radiuslocationid;
-  char* radiuslocationname;
-  char* locationname;
-  int radiusnasporttype;
-  int chillixml;     /* Send chilli specific XML along with WISPr */
-  int no_uamwispr;   /* Do not have Chilli return WISPr blocks */
+
   unsigned char nas_hwaddr[6];   /* Hardware address of NAS */
   
   int (*cb_getstate) (struct redir_t *redir, 
@@ -174,9 +195,8 @@ struct redir_t {
 			struct redir_conn_t *conn,
 			struct redir_httpreq_t *httpreq,
 			struct redir_socket_t *socket,
-			void *ctx);
-
-  void * cb_handle_url_ctx;
+			struct sockaddr_in *peer, 
+			redir_request *rreq);
 };
 
 struct redir_msg_data {
@@ -215,7 +235,7 @@ int redir_set_cb_getstate(struct redir_t *redir,
 int redir_main(struct redir_t *redir, int infd, int outfd, 
 	       struct sockaddr_in *address, 
 	       struct sockaddr_in *baddress,
-	       int isui, int forked);
+	       int isui, redir_request *rreq);
 
 int redir_json_fmt_redir(struct redir_conn_t *conn, bstring json, 
 			 char *userurl, char *redirurl, uint8_t *hismac);
