@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2009 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2010 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,6 +73,7 @@ typedef struct {
 } openssl_con;
 
 openssl_env * initssl();
+openssl_env * initssl_cli();
 int openssl_verify_peer(openssl_env *env, int mode);
 int openssl_use_certificate(openssl_env *env, char *file);
 int openssl_use_privatekey(openssl_env *env, char *file);
@@ -87,6 +88,7 @@ int openssl_write(openssl_con *con, char *b, int l, int t);
 void openssl_free(openssl_con *con);
 void openssl_env_free(openssl_env *env);
 int openssl_pending(openssl_con *con);
+int openssl_check_accept(openssl_con *c);
 
 #endif
 #endif
