@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2009 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,19 +27,7 @@
  *
  */
 
-#include "system.h"
-#include "tun.h"
-#include "ippool.h"
-#include "radius.h"
-#include "radius_wispr.h"
-#include "radius_chillispot.h"
-#include "redir.h"
-#include "syserr.h"
-#include "dhcp.h"
-#include "cmdline.h"
 #include "chilli.h"
-#include "options.h"
-#include "net.h"
 
 #define inaddr(x)    (((struct sockaddr_in *)&ifr->x)->sin_addr)
 #define inaddr2(p,x) (((struct sockaddr_in *)&(p)->x)->sin_addr)
@@ -63,9 +51,15 @@ int tun_discover(struct tun_t *this) {
     return -1;
   }
 
-  ic.ifc_buf = calloc((size_t)ic.ifc_len, 1);
+  if (!(ic.ifc_buf = calloc((size_t)ic.ifc_len, 1))) {
+    log_err(errno, "calloc(ic.ifc_buf)");
+    close(fd);
+    return -1;
+  }
+
   if (ioctl(fd, SIOCGIFCONF, &ic) < 0) {
     log_err(errno, "ioctl(SIOCGIFCONF)");
+    free(ic.ifc_buf);
     close(fd);
     return -1;
   }
@@ -203,6 +197,7 @@ int tun_discover(struct tun_t *this) {
     }
   }
 
+  free(ic.ifc_buf);
   close(fd);
   return 0;
 }
@@ -289,6 +284,8 @@ int tun_addaddr(struct tun_t *this, struct in_addr *addr,
   struct iovec iov;
   struct msghdr msg;
 
+  uint32_t idx;
+
   if (!this->addrs) /* Use ioctl for first addr to make ping work */
     return tun_setaddr(this, addr, dstaddr, netmask);
 
@@ -301,11 +298,13 @@ int tun_addaddr(struct tun_t *this, struct in_addr *addr,
   req.i.ifa_flags = 0;
   req.i.ifa_scope = RT_SCOPE_HOST; /* TODO or 0 */
 
-  if (tun_gifindex(this, &req.i.ifa_index)) {
+  if (tun_gifindex(this, &idx)) {
     log_err(errno,"tun_gifindex() failed");
     return -1;
   }
 
+  req.i.ifa_index = idx;
+
   tun_nlattr(&req.n, sizeof(req), IFA_ADDRESS, addr, sizeof(addr));
   tun_nlattr(&req.n, sizeof(req), IFA_LOCAL, dstaddr, sizeof(dstaddr));
 
@@ -474,6 +473,7 @@ int tuntap_interface(struct _net_interface *netif) {
   
   /* Set device flags. For some weird reason this is also the method
      used to obtain the network interface name */
+
   memset(&ifr, 0, sizeof(ifr));
 
   /* Tun device, no packet info */
@@ -501,7 +501,7 @@ int tuntap_interface(struct _net_interface *netif) {
     if ((nfd = socket (AF_INET, SOCK_DGRAM, 0)) >= 0) {
       strncpy(nifr.ifr_name, ifr.ifr_name, IFNAMSIZ);
       nifr.ifr_qlen = _options.txqlen;
-
+      
       if (ioctl(nfd, SIOCSIFTXQLEN, (void *) &nifr) >= 0) 
 	log_info("TX queue length set to %d", _options.txqlen);
       else 
@@ -536,7 +536,7 @@ int tuntap_interface(struct _net_interface *netif) {
       close(fd);
     }
   }
-
+  
   return 0;
   
 #elif defined(__FreeBSD__) || defined (__APPLE__) || defined (__OpenBSD__) || defined (__NetBSD__)
@@ -659,7 +659,6 @@ int tun_new(struct tun_t **ptun) {
 int tun_free(struct tun_t *tun) {
 
   if (tun->routes) {
-#warning fix this
     /*XXX: todo! net_delete_route(&tuntap(tun)); */
   }
 
@@ -682,8 +681,6 @@ struct tundecap {
   int idx;
 };
 
-
-
 static int tun_decaps_cb(void *ctx, void *packet, size_t length) {
   struct tundecap *c = (struct tundecap *)ctx;
 
@@ -692,6 +689,24 @@ static int tun_decaps_cb(void *ctx, void *packet, size_t length) {
     log_dbg("tun_decaps(idx=%d, len=%d)", tun(c->this,c->idx).ifindex, length);
   */
 
+#if defined(HAVE_NETFILTER_QUEUE) || defined(HAVE_NETFILTER_COOVA)
+  if (_options.uamlisten.s_addr != _options.dhcplisten.s_addr) {
+    struct pkt_iphdr_t *iph;
+
+    int ethhdr = (tun(c->this, c->idx).flags & NET_ETHHDR) != 0;
+
+    if (ethhdr)
+      iph = (struct pkt_iphdr_t *)((char *)packet + PKT_ETH_HLEN);
+    else
+      iph = (struct pkt_iphdr_t *)packet;
+
+    iph->daddr  = iph->daddr & ~(_options.mask.s_addr);
+    iph->daddr |= _options.dhcplisten.s_addr & _options.mask.s_addr;
+
+    chksum(iph);
+  }
+#endif
+
   if (c->idx > 0) {
     struct pkt_iphdr_t *iph = iphdr(packet);
 #ifdef ENABLE_NETNAT
@@ -734,8 +749,7 @@ int tun_decaps(struct tun_t *this, int idx) {
   }
 
   /*
-  if (this->debug)  
-    log_dbg("tun_decaps(%d) %s",status,tun(tun,idx).devname);
+  log_dbg("tun_decaps(%d) %s",status,tun(tun,idx).devname);
   */
 
    if (this->cb_ind)
@@ -825,11 +839,27 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
   }
 #endif
 
+#if defined(HAVE_NETFILTER_QUEUE) || defined(HAVE_NETFILTER_COOVA)
+  if (_options.uamlisten.s_addr != _options.dhcplisten.s_addr) {
+    struct pkt_iphdr_t *iph = iphdr(pack);
+
+    iph->saddr  = iph->saddr & ~(_options.mask.s_addr);
+    iph->saddr |= _options.uamlisten.s_addr & _options.mask.s_addr;
+
+    chksum(iph);
+  }
+#endif
+
   if (tun(tun, idx).flags & NET_ETHHDR) {
     uint8_t *gwaddr = _options.nexthop; /*tun(tun, idx).gwaddr;*/
     struct pkt_ethhdr_t *ethh = (struct pkt_ethhdr_t *)pack;
     /* memcpy(ethh->src, tun(tun, idx).hwaddr, PKT_ETH_ALEN); */
 
+    /*
+     * TODO: When using ieee8021q, the vlan tag has to be stripped
+     * off for the non-vlan WAN.
+     */
+
     if (gwaddr[0] == 0 && gwaddr[1] == 0 && gwaddr[2] == 0 && 
 	gwaddr[3] == 0 && gwaddr[4] == 0 && gwaddr[5] == 0) {
       /*  
@@ -865,21 +895,29 @@ int tun_encaps(struct tun_t *tun, uint8_t *pack, size_t len, int idx) {
   return tun_write(tun, pack, len, idx);
 }
 
-int tun_runscript(struct tun_t *tun, char* script) {
+int tun_runscript(struct tun_t *tun, char* script, int wait) {
   char saddr[TUN_ADDRSIZE];
   char smask[TUN_ADDRSIZE];
   char b[TUN_ADDRSIZE];
   struct in_addr net;
-  int status;
+  pid_t pid;
 
   net.s_addr = tuntap(tun).address.s_addr & tuntap(tun).netmask.s_addr;
 
-  if ((status = fork()) < 0) {
+  if ((pid = fork()) < 0) {
     log_err(errno, "fork() returned -1!");
     return 0;
   }
   
-  if (status > 0) { /* Parent */
+  if (pid > 0) { /* Parent */
+    if (wait) {
+      int status = 0;
+    again:
+      if (waitpid(pid, &status, 0) == -1) {
+	if (errno == EINTR) goto again;
+	log_err(errno, "waiting for %s", script);
+      }
+    }
     return 0;
   }
   
@@ -936,7 +974,14 @@ int tun_runscript(struct tun_t *tun, char* script) {
     exit(0);
   }
 
-  if (execl(script, script, tuntap(tun).devname, saddr, smask, (char *) 0) != 0) {
+  if (execl(
+#ifdef ENABLE_CHILLISCRIPT
+	    SBINDIR "/chilli_script", SBINDIR "/chilli_script", _options.binconfig, 
+#else
+	    script,
+#endif
+	    script, tuntap(tun).devname, saddr, smask, (char *) 0) != 0) {
+      
     log_err(errno, "execl(%s) did not return 0!",script);
     exit(0);
   }
