@@ -1,6 +1,6 @@
 /* 
  * Copyright (C) 2002, 2003, 2004, 2005 Mondru AB.
- * Copyright (C) 2007-2009 Coova Technologies, LLC. <support@coova.com>
+ * Copyright (C) 2007-2010 Coova Technologies, LLC. <support@coova.com>
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,7 +59,7 @@ int tun_set_cb_ind(struct tun_t *this, int (*cb_ind) (struct tun_t *tun, void *p
 
 int tun_setaddr(struct tun_t *this, struct in_addr *addr, struct in_addr *dstaddr, struct in_addr *netmask);
 
-int tun_runscript(struct tun_t *tun, char* script);
+int tun_runscript(struct tun_t *tun, char* script, int wait);
 
 net_interface *tun_nextif(struct tun_t *tun);
 int tun_name2idx(struct tun_t *tun, char *name);
