@@ -78,8 +78,8 @@ var chilliLibrary = { revision:'85' , apiVersion:'2.0' } ;
  *
  */
 
-if (!chilliController.host)
-var chilliController = { interval:30 , host:"192.168.1.1" , port:false , ident:'00' , ssl:false , uamService: false };
+if (!chilliController || !chilliController.host)
+var chilliController = { interval:30 , host:"1.0.0.1" , port:false , ident:'00' , ssl:false , uamService: false };
 
 /* Define clientState numerical code constants  */
 chilliController.stateCodes = { UNKNOWN:-1 , NOT_AUTH:0 , AUTH:1 , AUTH_PENDING:2 , AUTH_SPLASH:3 } ;
